/*
 * $Log$
 * Revision 1.2  2010/07/08 16:09:47  stuart
 * Modernize BMS.java
 *
 * Revision 1.1  2008/12/02 00:21:38  stuart
 * Include all in CVS
 *
 */
package config;

import java.io.*;
import java.util.*;
import bmsi.bms.SystemData;

/** Site specific configuration management.  Site configuration bundles are 
  resources which are customized for a site, and should <b>not</b> be
  updated when software is updated.  This configuration class is 
  installed separately from the main BMS java packages.  It uses the
  <code>bms.props</code> resource in the config package to find configurations
  for each subsystem.
    @author Stuart D. Gathman
    Copyright (C) 1999 Business Management Systems, Inc.
 */
public class BMS extends ResourceBundle {
  private final Map<String,ResourceBundle> subsys
    = new HashMap<String,ResourceBundle>();
  private Properties bms = new Properties();
  public static final BMS config = new BMS();

  /** Return the value of a resource key as an integer.  A convenience
  method that returns
    resources that extend <code>Number</code> as their integer value.
    It also attempts to parse String resources as a decimal integer.
    @param res	The ResourceBundle to query.
    @param key	The key.
    @return The integer value.
    @throws MissingResourceException when the key is not found
    	or could not be parsed as an integer.
   */
  public static int getValue(ResourceBundle res,String key)
    throws MissingResourceException {
    Object obj = res.getObject(key);
    if (obj instanceof Number)
      return ((Number)obj).intValue();
    try {
      return Integer.parseInt(obj.toString());
    }
    catch (NumberFormatException x) {
      throw new MissingResourceException("Number format: " + obj,
	res.getClass().getName(),key);
    }
  }

  private BMS() {
    InputStream is = getInputStream("bms.props");
    if (is != null)
      try {
	bms.load(new BufferedInputStream(is));
      }
      catch (IOException x) { }
  }

  private static InputStream getInputStream(String path) {
    InputStream is = null;
    try {
      if (path.startsWith(System.getProperty("path.separator")))
	is = new FileInputStream(path);
      else
	is = BMS.class.getResourceAsStream(path);
    }
    catch (IOException e) {
      System.err.println(path + ": configuration not found");
    }
    return is;
  }

  /** Return a site configuration bundle.  
  If a subsystem is not found, or the <code>bms.props</code> resource 
  is missing, the subsystem configuration is loaded as a resource
  in the config package with <code>.props</code> appended to the
  subsystem name.  
  <p>
  Values in the <code>bms.props</code> resource are pathnames or
  URLs.  A relative pathname is loaded as a resource in the config
  package.  An absolute pathname is loaded as a properties file
  in the local filesystem.  An URL with the <code>btas:</code> protocol
  is loaded as a SystemData file compatible with EDL code.
  Other URL's are loaded as text in the Java properties syntax.
  @param subsystem The subsystem key used to find a configuration bundle.
   */
  public static ResourceBundle getConfig(String subsystem)
    throws MissingResourceException {
    return (ResourceBundle)config.getObject(subsystem);
  }

  public Enumeration<String> getKeys() {
    /* Have to ensure are runtime that Properties contains only Strings */
    return new Enumeration<String>() {
      private final Enumeration keys = bms.keys();
      public boolean hasMoreElements() { return keys.hasMoreElements(); }
      public String nextElement() { return (String)keys.nextElement(); }
    };
  }

  public synchronized Object handleGetObject(String subsystem) {
    ResourceBundle p = subsys.get(subsystem);
    if (p == null) {
      String path = bms.getProperty(subsystem,subsystem + ".props");
      try {
	if (path.startsWith("btas:")) {
	  path = path.substring(5);
	  p = SystemData.getSystemData(path);
	}
	else {
	  InputStream is = getInputStream(path);
	  if (is != null)
	    p = new PropertyResourceBundle(is);
	}
      }
      catch (IOException x) { }
      if (p != null)
	subsys.put(subsystem,p);
    }
    return p;
  }

  /** List all accessible BMS configuration items. */
  public static void main(String[] argv) {
    SystemData.Desc desc = new SystemData.Desc();
    Enumeration e = config.getKeys();
    while (e.hasMoreElements()) {
      String key = (String)e.nextElement();
      System.out.println(key + '=' + config.bms.getProperty(key));
      try {
	ResourceBundle r = getConfig(key);
	Enumeration i = r.getKeys();
	final boolean hasDesc = r instanceof SystemData;
	while (i.hasMoreElements()) {
	  final String ik = (String)i.nextElement();
	  System.out.println("  " + ik + '=' + r.getObject(ik).toString());
	  if (hasDesc) {
	    try {
	      final String[] s = desc.getDesc(ik);
	      for (int k = 0; k < s.length; ++k)
		System.out.println("    " + s[k]);
	    }
	    catch (IOException x) {
	      System.out.println("    " + x.toString());
	    }
	  }
	}
      }
      catch (MissingResourceException m) {
        System.out.println("  *** MISSING ***");
      }
    }
  }
}
